package examples;

import gui.Viewer;

import java.util.Arrays;

import nenok.Architecture;
import nenok.Knowledgebase;
import nenok.LCFactory;
import nenok.lc.JoinTree;
import nenok.va.Domain;
import nenok.va.FiniteVariable;
import nenok.va.Valuation;
import nenok.va.Variable;
import sdf.SDF;

/**
 * 
 * 
 * @author Antoine De Groote
 */

public class Small {
    
	/**
	 * Main method:
	 * @param args Standard input arguments.
	 * <ul>
	 * 	<li>1. Argument: Execution mode (console | Interactive)</li>
	 * </ul>
	 */
    
    public static void main(String[] args) {
        
        // Verify number of arguments:
        if(args.length != 1) {
            System.out.println("Wrong number of Arguments.");
            System.out.println("Usage: Small (Interactive | Console)");
            System.exit(1);
        }
          
        /*
         * Variables:
         */
        
        FiniteVariable a = new FiniteVariable("A", null);
        FiniteVariable b = new FiniteVariable("B", null);
        FiniteVariable c = new FiniteVariable("C", null);
        FiniteVariable d = new FiniteVariable("D", null);
        FiniteVariable e = new FiniteVariable("E", null);
        
        /*
         * Valuations:
         */
        
        double inf = Double.POSITIVE_INFINITY;
            
    	double[][] dists1 = new double[][] { 
    			{ 0,3,2,inf },
    			{ 7,0,inf,1 },
    			{ 3,10,0,inf },
    			{ inf,inf,2,0 }
    	};
        SDF phi = new SDF(new FiniteVariable[] { a,b,c,d }, dists1);
        
        double[][] dists2 = new double[][] { 
        		{ 0,inf,3 },
        		{ 2,0,inf },
        		{ 5,1,0 }
        };
        SDF psi = new SDF(new FiniteVariable[] { c,d,e }, dists2);

        Valuation[] sdfs = new SDF[] { phi, psi };
                
        /*
         * Queries:
         */
        
        Domain[] queries = new Domain[] {
            new Domain(),
            new Domain(new Variable[] { a }),
            new Domain(new Variable[] { c,e }),
            new Domain(new Variable[] { a,b,c,d }),
            new Domain(new Variable[] { a,b,c,d,e })
        };
        
        try {
           
            /*
             * Local Computation:
             */
        	
        	Knowledgebase kb = Knowledgebase.create(sdfs, "Small Map");
            
            if(args[0].equalsIgnoreCase("Console")) {
            
                LCFactory factory = new LCFactory();
                factory.setArchitecture(Architecture.Idempotent);
                JoinTree tree = factory.create(kb, Arrays.asList(queries));
                tree.propagate();
                
                /*
                 * Query answering:
                 */
                
                System.out.println("Marginals of the Small-Map Example:\n");
                
                for(Domain query : queries) {
                	System.out.println("Query: "+query+"\n"+tree.answer(query));
                }
                            
            } else if(args[0].equalsIgnoreCase("Interactive")) {
            	            	
            	Viewer.display(kb);
                
            } else {
                System.out.println("Wrong Argument.");
                System.out.println("Usage: Small (Interactive | Console)");
            }
            
        } catch (Exception e1) {
            e1.printStackTrace();
            System.exit(1);
        }
    }
}
