package instances;

import semirings.Bottleneck;
import nenok.va.FiniteVariable;
import nenok.va.Idempotency;
import nenok.va.Scalability;
import nenok.va.Separativity;
import nenok.va.sr.OSRValuation;
import nenok.va.sr.SRValuation;
import nenok.va.sr.Semiring;

/**
 * Implementation of the valuation algebra that is induced by 
 * the Bottleneck semiring.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 574 $<br/>$LastChangedDate: 2008-03-26 14:50:48 +0100 (Mi, 26 Mrz 2008) $
 */

public class BottlePotential extends OSRValuation implements Idempotency {
	
	/**
	 * @see nenok.va.sr.SRValuation#create(nenok.va.FiniteVariable[], nenok.va.sr.Semiring[])
	 */
	
	@Override
	public SRValuation create(FiniteVariable[] vars, Semiring[] values) {
		return new BottlePotential(vars, cast(values));
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The semiring value for each possible configuration.
	 */
	
	public BottlePotential(FiniteVariable[] vars, Bottleneck[] values) {
		super(vars, values);
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The real number assigned to each configuration.
	 */
	
	public BottlePotential(FiniteVariable[] vars, int[] values) {
		super(vars, convert(values));
	}
	
	/**
     * @see nenok.va.Scalability#scale()
     */
    
    public Scalability scale() {
        return Idempotency.Implementor.getInstance().scale(this);
    }
	/**
	 * @see nenok.va.Separativity#inverse()
	 */
	
	public Separativity inverse() {
		return Separativity.Implementor.getInstance().inverse(this);
	}
	
	/**
	 * Casts an array of {@link Semiring} into an array of {@link Bottleneck}.
	 * @param values The element array to cast.
	 * @return An array of type {@link Bottleneck}.
	 */
	
	private static Bottleneck[] cast(Semiring[] values) {
		Bottleneck[] result = new Bottleneck[values.length];
		for (int i = 0; i < result.length; i++) {
			result[i] = (Bottleneck)values[i];
		}
		return result;
	}
	
	/**
	 * Converts an array of real numbers into semiring elements.
	 * @param values The input array of real numbers.
	 * @return A corresponding array of Bottleneck semiring elements.
	 */
	
	private static Bottleneck[] convert(int[] values) {
		Bottleneck[] elements = new Bottleneck[values.length];
		for (int i = 0; i < values.length; i++) {
			elements[i] = new Bottleneck(values[i]);
		}
		return elements;
	}
}
