package instances;

import semirings.Boolean;
import nenok.va.FiniteVariable;
import nenok.va.Idempotency;
import nenok.va.Scalability;
import nenok.va.Separativity;
import nenok.va.sr.OSRValuation;
import nenok.va.sr.SRValuation;
import nenok.va.sr.Semiring;

/**
 * Implementation of the valuation algebra that is induced by 
 * the Boolean semiring.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 574 $<br/>$LastChangedDate: 2008-03-26 14:50:48 +0100 (Mi, 26 Mrz 2008) $
 */

public class Indicator extends OSRValuation implements Idempotency {
	
	/**
	 * @see nenok.va.sr.SRValuation#create(nenok.va.FiniteVariable[], nenok.va.sr.Semiring[])
	 */
	
	@Override
	public SRValuation create(FiniteVariable[] vars, Semiring[] values) {
		return new Indicator(vars, cast(values));
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The semiring value for each possible configuration.
	 */
	
	public Indicator(FiniteVariable[] vars, Boolean[] values) {
		super(vars, values);
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The boolean constant assigned to each configuration.
	 */
	
	public Indicator(FiniteVariable[] vars, boolean[] values) {
		super(vars, convert(values));
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The integer (0 or 1) assigned to each configuration.
	 */
	
	public Indicator(FiniteVariable[] vars, int[] values) {
		super(vars, convert(values));
	}
	
	/**
	 * @see nenok.va.Separativity#inverse()
	 */
	
	public Separativity inverse() {
		return Idempotency.Implementor.getInstance().inverse(this);
	}
	
	/**
     * @see nenok.va.Scalability#scale()
     */
    
    public Scalability scale() {
        return Idempotency.Implementor.getInstance().scale(this);
    }
	
	/**
	 * Casts an array of {@link Semiring} into an array of {@link Boolean}.
	 * @param values The element array to cast.
	 * @return An array of type {@link Boolean}.
	 */
	
	private static Boolean[] cast(Semiring[] values) {
		Boolean[] result = new Boolean[values.length];
		for (int i = 0; i < result.length; i++) {
			result[i] = (Boolean)values[i];
		}
		return result;
	}
	
	/**
	 * Converts an array of numbers (0 or 1) into semiring elements.
	 * @param values The input array of real numbers.
	 * @return A corresponding array of Boolean semiring elements.
	 */
	
	private static Boolean[] convert(boolean[] values) {
		Boolean[] elements = new Boolean[values.length];
		for (int i = 0; i < values.length; i++) {
			elements[i] = new Boolean(values[i]);
		}
		return elements;
	}
	
	/**
	 * Converts an array of real numbers into semiring elements.
	 * @param values The input array of real numbers.
	 * @return A corresponding array of Boolean semiring elements.
	 */
	
	private static Boolean[] convert(int[] values) {
		Boolean[] elements = new Boolean[values.length];
		for (int i = 0; i < values.length; i++) {
			elements[i] = new Boolean(values[i]);
		}
		return elements;
	}
}
