package instances;

import semirings.MaxTropical;
import nenok.va.FiniteVariable;
import nenok.va.sr.OSRValuation;
import nenok.va.sr.SRValuation;
import nenok.va.sr.Semiring;

/**
 * Probability potential are an example of a valuation algebra and this
 * class offers an according implementation. It is based on the semiring
 * framework and allows to build two different kinds of semiring instances.
 * With respect to the enum type defined in this class, either probability 
 * potentials over the arithmetic semiring or max-probability potentials
 * over the tropical semiring are created.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 574 $<br/>$LastChangedDate: 2008-03-26 14:50:48 +0100 (Mi, 26 Mrz 2008) $
 */

public class MaxPotential extends OSRValuation {
		
	/**
	 * @see nenok.va.sr.SRValuation#create(nenok.va.FiniteVariable[], nenok.va.sr.Semiring[])
	 */
	
	@Override
	public SRValuation create(FiniteVariable[] vars, Semiring[] values) {
		return new MaxPotential(vars, cast(values));
	}
	
	/**
	 * Constructor:
	 * @param var The single variable of this semiring valuation. 
	 * @param values The values assigned to each configuration.
	 */
	
	public MaxPotential(FiniteVariable var, int[] values) {
		super(new FiniteVariable[] {var}, convert(values));
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The semiring value for each possible configuration.
	 */
	
	public MaxPotential(FiniteVariable[] vars, MaxTropical[] values) {
		super(vars, values);
	}
		
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The value assigned to each configuration.
	 */
	
	public MaxPotential(FiniteVariable[] vars, int[] values) {
		super(vars, convert(values));
	}
		
	/**
	 * Casts an array of {@link Semiring} into an array of {@link Arithmetic}.
	 * @param values The element array to cast.
	 * @return An array of type {@link Arithmetic}.
	 */
	
	private static MaxTropical[] cast(Semiring[] values) {
		MaxTropical[] result = new MaxTropical[values.length];
		for (int i = 0; i < result.length; i++) {
			result[i] = (MaxTropical)values[i];
		}
		return result;
	}
	
	/**
	 * Converts an array of real numbers into semiring elements of the given type.
	 * @param values The input array of real numbers.
	 * @param type The type of semiring values that are created.
	 * @return A corresponding array of the Arithmetic semiring elements.
	 */
	
	private static MaxTropical[] convert(int[] values) {
		MaxTropical[] elements = new MaxTropical[values.length];
		for (int i = 0; i < values.length; i++) {
			elements[i] = new MaxTropical(values[i]);
		}
		return elements;
	}
}
