package semirings;

import nenok.va.sr.ICSemiring;
import nenok.va.sr.Optimization;
import nenok.va.sr.Semiring;
import nenok.va.sr.SeparativeSemiring;

/**
 * Implementation of the Bottleneck semiring. This semiring is a strictly monotonic,
 * totally ordered, idempotent C-semiring. Therefore, it qualifies for modeling 
 * optimization problems and consequently, it extends {@link Optimization} 
 * that contains the appropriate ordering relation.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 573 $<br/>$LastChangedDate: 2008-03-26 14:50:22 +0100 (Mi, 26 Mrz 2008) $
 */

public class Bottleneck implements ICSemiring, Optimization {
	
	private int value;
	
	/**
	 * Constructor:
	 * @param value The value of the semiring element.
	 */
	
	public Bottleneck(int value) {
		this.value = value;
	}
	
	/**
	 * @see nenok.va.sr.Semiring#add(nenok.va.sr.Semiring)
	 */
	
	public Semiring add(Semiring semiring) {
        if(!(semiring instanceof Bottleneck)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Bottleneck.class+".");
        }
		return new Bottleneck(Math.max(((Bottleneck)semiring).value, value));
	}

	/**
	 * @see nenok.va.sr.Semiring#multiply(nenok.va.sr.Semiring)
	 */
	
	public Semiring multiply(Semiring semiring) {
        if(!(semiring instanceof Bottleneck)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Bottleneck.class+".");
        }
		return new Bottleneck(Math.min(((Bottleneck)semiring).value, value));
	}
	
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(!(o instanceof Bottleneck)) {
            return false;
        }
        return ((Bottleneck)o).value == value;
    }
    
    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    
    public int compareTo(Object semiring) {
        if(!(semiring instanceof Bottleneck)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Bottleneck.class+".");
        }
    	return Optimization.Implementor.getInstance().compare((Optimization)semiring, this);
    }
    
    /**
     * @see nenok.va.sr.SeparativeSemiring#inverse()
     */
    
    public SeparativeSemiring inverse() {
    	return ICSemiring.Implementor.getInstance().inverse(this);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return value+"";
    }
}
