package examples;

import java.util.Arrays;

import nenok.Architecture;
import nenok.LCFactory;
import nenok.lc.JoinTree;
import nenok.va.Domain;
import gui.Viewer;

/**
 * Demo application for the use of probability potentials.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 586 $<br/>$LastChangedDate: 2008-03-26 16:20:24 +0100 (Mi, 26 Mrz 2008) $
 */

public class Demo {

	/**
	 * Main method:
	 * @param args Standard input arguments.
	 * <ul>
	 * 	<li>1. Argument: Name of the example to be executed.</li>
	 * 	<li>2. Argument: Execution mode (console | Interactive)</li>
	 * </ul>
	 */
	
	public static void main(String[] args) {
		
		if(args.length != 2) {
			System.out.println("Missing argument.");
			System.out.println("Usage: Demo example (Interactive | Console)");			
			System.exit(1);
		}

        /*
         * Get input data:
         */
        
        Input input = null;
        
		if(args[0].equalsIgnoreCase("Asia")) {
			input = Input.getAsiaInput();
		} else if(args[0].equalsIgnoreCase("Dog")) {
			input = Input.getDogInput();
		} else if(args[0].equalsIgnoreCase("Studfarm")) {
			input = Input.getStudfarmInput();		
		} else if(args[0].equalsIgnoreCase("Montyhall")) {
			input = Input.getMontyHallInput();	
		} else if(args[0].equalsIgnoreCase("Earthquake")) {
			input = Input.getEarthquakeInput();	
		} else if(args[0].equalsIgnoreCase("Cancer")) {
			input = Input.getCancerInput();	
		} else if(args[0].equalsIgnoreCase("Oil")) {
			input = Input.getOilInput();
		} else if(args[0].equalsIgnoreCase("Elisa")) {
			input = Input.getElisaInput();
		} else {
			throw new IllegalArgumentException("Example "+args[0]+" not found.");
		}
		
        /*
         * Local Computation:
         */
				
        try {
        
            if(args[1].equalsIgnoreCase("Console")) {
            	
                LCFactory factory = new LCFactory();
                factory.setArchitecture(Architecture.Shenoy_Shafer);
           
                JoinTree tree = factory.create(input.knowledgebase, Arrays.asList(input.queries));
                tree.propagate(true);
                
                System.out.println("Marginals of the "+args[0]+" example:");
                System.out.println();
                
                for(Domain query : input.queries) {
                    System.out.println("Query: "+query);
                    System.out.println(tree.answer(query));
                }     
                
            } else if(args[1].equalsIgnoreCase("Interactive")) {
                
            	Viewer.display(input.knowledgebase);
            	
            } else {
            	
    			System.out.println("Wrong second argument.");
    			System.out.println("Usage: Demo example (Interactive | Console)");			
    			System.exit(1);
            }
            
        } catch (Exception e1) {
            e1.printStackTrace();
            System.exit(1);
        }
	}
	
}
