package ppotentials;

import nenok.va.sr.Semiring;
import nenok.va.sr.SeparativeSemiring;

/**
 * Random implementation of the Arithmetic semiring.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 568 $<br/>$LastChangedDate: 2008-03-26 14:42:35 +0100 (Mi, 26 Mrz 2008) $
 */

public class Arithmetic implements SeparativeSemiring {
            
    private double value;
    
    //Distance for equality checks:
    private static final double EPSILON = 0.0000001;
        
    /**
     * Constructor:
     * @param value The value of this semiring element.
     */
    
    public Arithmetic(double value) {
    	if(value < 0) {
    		throw new IllegalArgumentException("Only non-negative values accepted.");
    	}
        this.value = value;
    }

    /**
     * @see nenok.va.sr.Semiring#add(nenok.va.sr.Semiring)
     */
    
    public Semiring add(Semiring semiring) {
        if(!(semiring instanceof Arithmetic)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Arithmetic.class+".");
        }
        return new Arithmetic(value + ((Arithmetic)semiring).value);
    }

    /**
     * @see nenok.va.sr.Semiring#multiply(nenok.va.sr.Semiring)
     */
    
    public Semiring multiply(Semiring semiring) {
        if(!(semiring instanceof Arithmetic)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Arithmetic.class+".");
        }
        return new Arithmetic(value * ((Arithmetic)semiring).value);
    }
    
    /**
     * @see nenok.va.sr.SeparativeSemiring#inverse()
     */

    public SeparativeSemiring inverse() {
        return (this.value == 0)? new Arithmetic(0) : new Arithmetic(1 / value);
    }    
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(!(o instanceof Arithmetic)) {
            return false;
        }
        return numericTest(((Arithmetic)o).value, value);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return String.format("%.3f", value);
    }

    /*
     * Getter &amp; Setter:
     */
    
    /**
     * @return The probability object's value.
     */
    
    public double getValue() {
        return this.value;
    }
    
    /**
     * This test compares two numeric double values on equality.
     * They are equal if their distance is smaller that the EPSILON constant.
     * @param d1 The first value.
     * @param d2 The second value.
     * @return <code>true</code>, if the two values are equal.
     */
    
    private static boolean numericTest(double d1, double d2) {
        double distance = Math.abs(d1 - d2);
        return !(distance > EPSILON);
    }
}
