package ppotentials;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import nenok.parser.KB_Parser;
import nenok.parser.ParserException;
import nenok.parser.ResultSet;
import nenok.va.Domain;
import nenok.va.FiniteVariable;
import nenok.va.Valuation;

/**
 * A NENOK parser implementation that parses DNET files.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 568 $<br/>$LastChangedDate: 2008-03-26 14:42:35 +0100 (Mi, 26 Mrz 2008) $
 */

public class DNET_Parser implements KB_Parser {

	/**
	 * @see nenok.parser.Parser#accept(java.io.File)
	 */
	
	public boolean accept(File file) {
		return file.getName().endsWith(".dnet") || file.getName().endsWith(".dne");
	}
	
	/**
	 * @see nenok.parser.KB_Parser#parse(java.io.File)
	 */
	
	public ResultSet parse(File file) throws ParserException {
		try {
						
			BufferedReader in = new BufferedReader(new FileReader(file));
			StreamTokenizer tokenizer = new StreamTokenizer(in);
			
			tokenizer.resetSyntax();
			tokenizer.parseNumbers();
			tokenizer.wordChars('0', '9');
			tokenizer.wordChars('A', 'Z');
		    tokenizer.wordChars('a', 'z'); 
		    tokenizer.wordChars('_', '_'); 
		    tokenizer.wordChars('-', '-'); 
		    tokenizer.wordChars('=', '='); 
		    tokenizer.wordChars('@', '@'); 
			tokenizer.slashSlashComments(true);
			
			/*
			 * Run tokenizer:
			 */
			
			int type;
			ArrayList<String> tokens = new ArrayList<String>();
		    while ((type = tokenizer.nextToken()) != StreamTokenizer.TT_EOF) {
		    	switch (type) {
					case StreamTokenizer.TT_WORD:
						 tokens.add(tokenizer.sval);
						 break;
					
					case StreamTokenizer.TT_NUMBER:
						 tokens.add(Double.toString(tokenizer.nval));
						 break;
				default:
					break;
				}
		    	
		    	if(tokenizer.ttype == ';' || tokenizer.ttype == ',' || 
		    	   tokenizer.ttype == '{' || tokenizer.ttype == '}' ||
		    	   tokenizer.ttype == '(' || tokenizer.ttype == ')' ||
		    	   tokenizer.ttype == '[' || tokenizer.ttype == ']') {
		    		tokens.add(new String(Character.toChars(tokenizer.ttype)));
		    		continue;
		    	}		    	
		    }
		    
		    
			/*
			 * Parsing process:
			 */
		    
			Hashtable<String, FiniteVariable> variables = new Hashtable<String, FiniteVariable>();
			ArrayList<Potential> valuations = new ArrayList<Potential>();
			Iterator<String> iter = tokens.iterator();
			
			String current;
			if(!(current = iter.next()).equals("bnet")) {
				throw new DNETException(current, "bnet");
			}
			
			iter.next(); // Network title
			
			if(!(current = iter.next()).equals("{")) {
				throw new DNETException(current, "{");
			}
						
			/*
			 * Parse Variables & Valuations
			 */
			
			Hashtable<String, FiniteVariable> inherits = new Hashtable<String, FiniteVariable>();
			
			while(!current.equals("}")) {
				
				/*
				 * Template entries:
				 */
				
				if(current.equals("define")) {
					
					if(!(current = iter.next()).equals("node")) {
						throw new DNETException(current, "node");
					}
					
					/*
					 * Template name:
					 */
					
					String name = iter.next();
					
					if(!(current = iter.next()).equals("{")) {
						throw new DNETException(current, "{");
					}
					
					current = iter.next();
					
					while(true) {
						
						/*
						 * Template states:
						 */
						
						if(current.equals("states")) {
							ArrayList<String> stateList = new ArrayList<String>();
							if(!(current = iter.next()).equals("=")) {
								throw new DNETException(current, "=");
							}
							if(!(current = iter.next()).equals("(")) {
								throw new DNETException(current, "(");
							}
							do {
								stateList.add(iter.next());
							} while ((current = iter.next()).equals(","));
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							inherits.put(name, new FiniteVariable(name, stateList.toArray(new String[] {})));
							current = iter.next();
						}
						
						/*
						 * Finalize Template:
						 */
						
						else if(current.equals("}")) {
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							current = iter.next();
							break;
						}
						
						else {
							int counter = 0;
							current = iter.next();
							while(!(current.equals(";") && counter == 0)) {
								if(current.equals("{")) {
									counter++;
								}
								if(current.equals("}")) {
									counter--;
								}
								current = iter.next();
							}
							current = iter.next();
						}
					}
				}
				
				/*
				 * Node entries:
				 */
				
				else if(current.equals("node")) {
					
					/*
					 * Variable name:
					 */
					
					String name = iter.next(); 
					current = iter.next();
					
					/*
					 * Inheritance:
					 */
					
					String inherit = null;
					if(current.equals("(")) {
						inherit = iter.next();
						if(!(current = iter.next()).equals(")")) {
							throw new DNETException(current, ")");
						}
						current = iter.next();
					} 
					
					/*
					 * Content:
					 */
					
					if(!current.equals("{")) {
						throw new DNETException(current, "{");
					}
					current = iter.next();
					
					FiniteVariable[] cond = null;
					double[] probabilities = null;
					ArrayList<String> stateList = new ArrayList<String>();
										
					while(true) {
																		
						/*
						 * Symbolic States:
						 */
												
						if(current.equals("states")) {
							if(!(current = iter.next()).equals("=")) {
								throw new DNETException(current, "=");
							}
							if(!(current = iter.next()).equals("(")) {
								throw new DNETException(current, "(");
							}
							do {
								stateList.add(iter.next());
							} while ((current = iter.next()).equals(","));
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							current = iter.next();
						}
						
						/*
						 * Numeric States:
						 */
						
						else if(current.equals("numstates")) {
							if(!(current = iter.next()).equals("=")) {
								throw new DNETException(current, "=");
							}
							int nbStates = (int)Double.parseDouble(iter.next());
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							for (int i = 0; i < nbStates; i++) {
								stateList.add(i+"");
							}
							current = iter.next();
						}
						
						/*
						 * Discrete vs. continous variables:
						 */
						
						else if(current.equals("discrete")) {
							if(!(current = iter.next()).equals("=")) {
								throw new DNETException(current, "=");
							}
							if(!(current = iter.next()).equalsIgnoreCase("true")) {
								throw new DNETException("Continous variables are not supported by NENOK.");
							}
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							current = iter.next();
						}
						
						/*
						 * Conditionals:
						 */
						
						else if(current.equals("parents")) {
							ArrayList<FiniteVariable> parentList = new ArrayList<FiniteVariable>();
							if(!(current = iter.next()).equals("=")) {
								throw new DNETException(current, "=");
							}
							if(!(current = iter.next()).equals("(")) {
								throw new DNETException(current, "(");
							}
							current = iter.next();						
							while(!current.equals(")")) {
								FiniteVariable parent = variables.get(current);
								if(parent == null) {
									throw new DNETException("Variable "+current+" not yet defined.");
								}
								parentList.add(parent);
								current = iter.next();
								if(current.equals(",")) {
									current = iter.next();
								}
							}
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							cond = parentList.toArray(new FiniteVariable[] {});
							current = iter.next(); 
						}
						
						/*
						 * Probabilities:
						 */
						
						else if(current.equals("probs")) {
							ArrayList<Double> probList = new ArrayList<Double>();
							if(!(current = iter.next()).equals("=")) {
								throw new DNETException(current, "=");
							}
							if(!(current = iter.next()).equals("(")) {
								throw new DNETException(current, "(");
							}
							while(!(current = iter.next()).equals(";")) {
								if(current.equals("(") || current.equals(")")) {
									continue;
								}
								if(current.equals(",")) {
									continue;
								}
								if(current.equals("@undef")) {
									throw new DNETException("Undefined probability values are not supported by NENOK.");
								} else if(current.equals("@imposs")) {
									probList.add(0.0);
								} else {
									Double p = Double.parseDouble(current);
									current = iter.next();
									if(current.startsWith("E")) {							
										int exp = Integer.parseInt(current.substring(2, current.length()));
										p = Math.pow(p, exp);
										current = iter.next();
									} 
	
									probList.add(p);
								}
							}
							current = iter.next();
							probabilities = new double[probList.size()];
							for (int i = 0; i < probabilities.length; i++) {
								probabilities[i] = probList.get(i).doubleValue();
							}
						}
						
						/*
						 * Finalize:
						 */
						
						else if(current.equals("}")) {
							if(!(current = iter.next()).equals(";")) {
								throw new DNETException(current, ";");
							}
							
							/*
							 * Make Variable:
							 */
							
							if(!stateList.isEmpty()) {
								variables.put(name, new FiniteVariable(name, stateList.toArray(new String[] {})));
							} else {
								String[] states = inherits.get(inherit).getFrame();
								variables.put(name, new FiniteVariable(name, states));
							}
							
							/*
							 * Make Valuation:
							 */
							
							if(probabilities != null && cond != null) {
								valuations.add(new Potential(variables.get(name), cond, probabilities));
							}
							
							current = iter.next();
							break;
						}
						
						else {
							int counter = 0;
							current = iter.next();
							while(!(current.equals(";") && counter == 0)) {
								if(current.equals("{")) {
									counter++;
								}
								if(current.equals("}")) {
									counter--;
								}
								current = iter.next();
							}
							current = iter.next();
						}
					}
				} 
				
				/*
				 * Ignore all other entries:
				 */
				
				else {
					int counter = 0;
					current = iter.next();
					while(!(current.equals(";") && counter == 0)) {
						if(current.equals("{")) {
							counter++;
						}
						if(current.equals("}")) {
							counter--;
						}
						current = iter.next();
					}
					
					/*
					 * Finalize file:
					 */
					
					current = iter.next();
					if(current.equals("}")) {
						if(!(current = iter.next()).equals(";")) {
							throw new DNETException(current, ";");
						}
						break;
					}
				}
			}
						
			in.close();
			return new ResultSet(valuations.toArray(new Valuation[valuations.size()]), new Domain[] {});
			
		} catch (FileNotFoundException e) {
			throw new ParserException("Parsing Error - File not found.", e);
		} catch (IOException e) {
			throw new ParserException("Parsing Error", e);
		}
	}
	
	/**
	 * Exception class for DNET parsing.
	 * 
	 * @author Marc Pouly
	 * @version 1.0
	 */
	
	public class DNETException extends ParserException {
		
		/**
		 * Constructor:
		 * @param message The error message.
		 */
		
		public DNETException(String message) {
			super(message);
		}
		
		/**
		 * Constructor:
		 * @param found The token that has been found.
		 * @param expected The expected token.
		 */
		
		public DNETException(String found, String expected) {
			super("Parsing Exception: Expected token is "+expected+", found "+found+".");
			System.out.println("Parsing Exception: Expected token is "+expected+", found "+found+".");
		}	
	}
}
