package nenok;

import nenok.adapt.Labeled;
import nenok.va.Domain;
import nenok.va.Identity;
import nenok.va.Valuation;

/**
 * Knowledgebase subclass for local valuations.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 550 $<br/>$LastChangedDate: 2008-03-26 14:14:27 +0100 (Mi, 26 Mrz 2008) $
 */

public class LocalKB extends Knowledgebase {
	
	private Class type;
	private Domain domain;
	private final Valuation[] valuations;
	
	/**
	 * Constructor:
	 */
	
	protected LocalKB(Valuation[] vals, String name) {
		super(name);
		this.valuations = vals;
	}
		
	/**
	 * @see nenok.Knowledgebase#getValuations()
	 */

	@Override
	public Valuation[] getValuations() {
		return valuations;
	}
	
	/**
	 * @see nenok.Knowledgebase#toArray()
	 */
	
	@Override
	public Labeled[] toArray() {
		return valuations;
	}
		
	/**
	 * @see nenok.Knowledgebase#getDomain()
	 */

	@Override
	public Domain getDomain() {
		if(domain == null) {
			domain = Domain.totalDomain(valuations);
		}
		return domain;
	}
	
	/**
	 * @see nenok.Knowledgebase#getType()
	 */

	@Override
	public Class getType() {
		if(type == null) {
	    	for(Valuation val : valuations) {
	    		if(!(val instanceof Identity)) {
	        		type = val.getClass();
	        		break;
	    		}
	    	}
		}
		return type;
	}
	
	/**
	 * @see nenok.Knowledgebase#size()
	 */
	
	@Override
	public int size() {
		return valuations.length;
	}
}
