package nenok.adapt;

import nenok.lc.LCException;
import nenok.va.Domain;

/**
 * An adapter class that allows to process local and remote valuations in a 
 * uniform manner. Additionally, this class counts the number of executed 
 * operations.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 552 $<br/>$LastChangedDate: 2008-03-26 14:17:19 +0100 (Mi, 26 Mrz 2008) $
 */

public interface Adapter {
	
	/**
	 * Inverse operation.
	 * @param factor The factor whose inverse has to be computed.
	 * @return The inverse of the factor.
	 * @throws LCException Generic local computation exception.
	 */
	
	public Labeled inverse(Labeled factor) throws LCException;
	
	/**
	 * Combination operation.
	 * @param factor The first factor of the combination. Important: in case of a remotely processed 
	 * combination, this argument determined the processor which executes the combination.
	 * @param factors The remaining factors to be combined.
	 * @return The combination of all factors given as argument. 
	 * The implementation of this combination should respect the array's order.
	 * @throws LCException Generic local computation exception.
	 */
	
	public Labeled combine(Labeled factor, Labeled... factors) throws LCException;
	
    /**
     * Marginalization operation.
     * @param factor The factor to be marginalized.
     * @param dom The domain onto the marginalization is performed.
     * @return The marginal of the argument relativ to the given domain.
	 * @throws LCException Generic local computation exception.
     */
	
	public Labeled marginalize(Labeled factor, Domain dom) throws LCException;
	
    /**
     * @return Returns the number of combinations.
     */
    
    public int getCombinations();
    
    /**
     * @return Returns the number of inverses.
     */
    
    public int getInverses();
    
    /**
     * @return Returns the number of marginalizations.
     */
    
    public int getMarginalizations();
}
