package nenok.adapt;

import nenok.lc.LCException;
import nenok.va.Domain;
import nenok.va.Separativity;
import nenok.va.VAException;
import nenok.va.Valuation;

/**
 * Refined adapter class for the processing of remote valuations.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 552 $<br/>$LastChangedDate: 2008-03-26 14:17:19 +0100 (Mi, 26 Mrz 2008) $
 */

public class LocalAdapter implements Adapter {
    
    private int combinations, marginalizations, inverses;
        
    /**
     * @see nenok.adapt.Adapter#combine(nenok.adapt.Labeled, nenok.adapt.Labeled[])
     */
    
    public Labeled combine(Labeled factor, Labeled... factors) throws LCException {
    	Valuation result = (Valuation)factor;
    	for (int i = 0; i < factors.length; i++) {
			result = result.combine((Valuation)factors[i]);
			combinations++;
		}
        return result;
    }
       
    /**
     * @see nenok.adapt.Adapter#marginalize(nenok.adapt.Labeled, nenok.va.Domain)
     */
    
    public Labeled marginalize(Labeled factor, Domain dom) throws VAException {
        marginalizations++;
        return ((Valuation)factor).marginalize(dom);
    }
    
    /**
     * @see nenok.adapt.Adapter#inverse(nenok.adapt.Labeled)
     */
    
    public Labeled inverse(Labeled factor) {
    	inverses++;
        return ((Separativity)factor).inverse();
    }
    
    /**
     * @see nenok.adapt.Adapter#getCombinations()
     */
    
    public int getCombinations() {
    	return combinations;
    }
    
    /**
     * @see nenok.adapt.Adapter#getInverses()
     */
    
    public int getInverses() {
        return inverses;
    }
    
    /**
     * @see nenok.adapt.Adapter#getMarginalizations()
     */
    
    public int getMarginalizations() {
        return marginalizations;
    }
}
