package nenok.adapt;

import nenok.Services;
import nenok.lc.LCException;
import nenok.net.IdentityLocator;
import nenok.net.Locator;
import nenok.net.processor.Processor;
import nenok.va.Domain;

/**
 * Refined adapter class for the processing of remote valuations.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 552 $<br/>$LastChangedDate: 2008-03-26 14:17:19 +0100 (Mi, 26 Mrz 2008) $
 */

public class RemoteAdapter implements Adapter {
	
    private int combinations, marginalizations, inverses, costs;

	/**
	 * @see nenok.adapt.Adapter#combine(nenok.adapt.Labeled, nenok.adapt.Labeled[])
	 */
	
    public Labeled combine(Labeled factor, Labeled... factors) throws LCException {
		
    	if(factors.length == 0) {
			return factor;
		}
		
		try {
		
			Locator loc = (Locator)factor;
			Processor proc = Services.findProcessor(loc.getPID());
			Locator[] args = new Locator[factors.length];
			for (int i = 0; i < args.length; i++) {
				args[i] = (Locator)factors[i];
			}
			
			// Compute communication costs:
			for (int i = 0; i < args.length; i++) {
				if(!loc.getPID().equals(args[0].getPID())) {
					costs += args[0].getWeight();
				}
			}
			
			return proc.combine(loc, args);
		
		} catch (Exception e) {
			throw new LCException(e);
		}
	}

	/**
	 * @see nenok.adapt.Adapter#inverse(nenok.adapt.Labeled)
	 */
	
	public Labeled inverse(Labeled factor) throws LCException {
		if(factor instanceof IdentityLocator) {
			return factor;
		}
		try {
			Locator loc = (Locator)factor;
			Processor proc = Services.findProcessor(loc.getPID());
			return proc.inverse((Locator)factor);
		} catch (Exception e) {
			throw new LCException(e);
		}
	}

	/**
	 * @see nenok.adapt.Adapter#marginalize(nenok.adapt.Labeled, nenok.va.Domain)
	 */
	
	public Labeled marginalize(Labeled factor, Domain dom) throws LCException {
		if(factor instanceof IdentityLocator) {
			return factor;
		}
		try {
			Locator loc = (Locator)factor;
			Processor proc = Services.findProcessor(loc.getPID());
			return proc.marginalize(loc, dom);
		} catch (Exception e) {
			throw new LCException(e);
		}
	}

	/**
	 * @return The communication costs that came up until now.
	 */
	
	public double getCommunicationCosts() {
		return costs;
	}
	
    /**
     * @see nenok.adapt.Adapter#getCombinations()
     */
    
    public int getCombinations() {
    	return combinations;
    }
    
    /**
     * @see nenok.adapt.Adapter#getInverses()
     */
    
    public int getInverses() {
        return inverses;
    }
    
    /**
     * @see nenok.adapt.Adapter#getMarginalizations()
     */
    
    public int getMarginalizations() {
        return marginalizations;
    }
}
