package nenok.alloc;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import nenok.lc.Node;
import nenok.net.IdentityLocator;
import nenok.net.Locator;
import net.jini.id.Uuid;

/**
 * This is a random processor allocation model. From the existing set of processor,
 * we assign a random processor to all node that initially posses an identity locator.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 553 $<br/>$LastChangedDate: 2008-03-26 14:18:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class Random implements Allocator {
	
	private List<Uuid> pids;	
	private static final java.util.Random generator = new java.util.Random();
        
    /**
     * Constructor:
     * @param pids The set of existing processors.
     */
    
    public Random(Set<Uuid> pids) {
        this.pids = new ArrayList(pids);
    }

    /**
     * @see nenok.alloc.Allocator#allocate(nenok.lc.Node)
     */
    
    public void allocate(Node node) throws RemoteException {
               
    	Locator loc = (Locator)node.getContent();
    	
        if(loc instanceof IdentityLocator) {
        	
        	Uuid pid = pids.get(generator.nextInt(pids.size()));
        	((IdentityLocator)loc).setPID(pid);
        	
        }
    }
    
    /**
     * @see nenok.alloc.Allocator#getName()
     */
    
    public String getName() {
        return "Random Allocation";
    }
}
