package nenok.constr.vvll;

import nenok.va.Variable;

/**
 * Objects of this extension of the {@link VariableContainer} class
 * will be stored within a heap. Therefore, the {@link Comparable} interface
 * is implemented additionally. The <code>compareTo(Object o)</code> is based
 * on a cost value representing the costs to eliminate the variable of this
 * container.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 554 $<br/>$LastChangedDate: 2008-03-26 14:19:57 +0100 (Mi, 26 Mrz 2008) $
 */

public class VariableContainerHeap extends VariableContainer implements Comparable {
	
	private int cost;
	
	/**
	 * Constructor:
	 * Calls super constructor and initializes costs to 0.
	 * @param var The variable of this container.
	 */
	
	public VariableContainerHeap(Variable var) {
		super(var);
		this.cost = 0;
	}

	/**
	 * Sets the variable containers's cost value.
	 * @param value The new cost of the variable.
	 */
	
	public void setCost(int value) {
		this.cost = value;
	}
	
	/*
	 * Comparable interface:
	 */
	
	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	
	public int compareTo(Object o) {
		VariableContainerHeap vc = (VariableContainerHeap)o;
		if (this.cost == vc.cost) {
			return 0;	
		}
		if (this.cost > vc.cost) {
			return 1;
		}
		return -1;
	}	
}














