package nenok.lc.hugin;

import java.util.Collection;

import nenok.Knowledgebase;
import nenok.Utilities;
import nenok.adapt.Labeled;
import nenok.constr.Algorithm;
import nenok.constr.ConstrException;
import nenok.lc.JoinTree;
import nenok.lc.LCException;
import nenok.lc.Node;
import nenok.va.Domain;
import nenok.va.Separativity;

/**
 * The join tree corresponding to the HUGIN architecture.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 557 $<br/>$LastChangedDate: 2008-03-26 14:23:32 +0100 (Mi, 26 Mrz 2008) $
 */

public class HJoinTree extends JoinTree {
	    
	/**
	 * Constructor:
     * @param kb The knowledgebase from which this jointree is constructed.
     * @param queries The set of queries that are covered by this join tree.
     * @param algo The construction algorithm.
     * @throws ConstrException Exceptions throws during construction process.
     */
	
	public HJoinTree(Knowledgebase kb, Collection<Domain> queries, Algorithm algo) throws ConstrException {
        super(kb, queries, algo);
    }

	/** 
     * @see nenok.lc.JoinTree#verify(java.lang.Class)
	 */
    
    public boolean verify(Class type) {
        return Utilities.interfaceOf(Separativity.class, type);
    }

    /** 
     * @see nenok.lc.JoinTree#getArchitecture()
	 */
	 
	public String getArchitecture() {
		return "HUGIN";
	}

	/** 
     * @see nenok.lc.JoinTree#scale()
	 */
	
	public void scale() throws LCException {
        
		/*
         * Compute inverse (scaling factor):
         */
        
        Labeled result = root.getContent();
        
        if(result.label().size() > 0) {
            result = adapter.marginalize(root.getContent(), Domain.EMPTY);
        }
        
        result = adapter.inverse(result);
        
        /*
         * Combine inverse with root content:
         */
        
        this.getRoot().setContent(adapter.combine(getRoot().getContent(), result));
	}

	/**
     * @see nenok.lc.JoinTree#createNode()
	 */
    
	public Node createNode() {
        return new HNode(adapter);
	}
}
