package nenok.lc.id;

import java.util.Collection;

import nenok.Knowledgebase;
import nenok.Utilities;
import nenok.constr.Algorithm;
import nenok.constr.ConstrException;
import nenok.lc.JoinTree;
import nenok.lc.Node;
import nenok.va.Domain;
import nenok.va.Idempotency;
import nenok.va.VAException;

/**
 * The join tree corresponding to the idempotent architecture.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 557 $<br/>$LastChangedDate: 2008-03-26 14:23:32 +0100 (Mi, 26 Mrz 2008) $
 */

public class IDJoinTree extends JoinTree {
	    
	/**
	 * Constructor:
     * @param kb The knowledgebase from which this jointree is constructed.
     * @param queries The set of queries that are covered by this join tree.
     * @param algo The construction algorithm.
     * @throws ConstrException Exceptions throws during construction process.
     */
	
	public IDJoinTree(Knowledgebase kb, Collection<Domain> queries, Algorithm algo) throws ConstrException {
        super(kb, queries, algo);
    }

    /** 
     * @see nenok.lc.JoinTree#verify(java.lang.Class)
     */
    
    public boolean verify(Class type) {
        return Utilities.interfaceOf(Idempotency.class, type);
    }

    /** 
     * @see nenok.lc.JoinTree#getArchitecture()
	 */
	 
	public String getArchitecture() {
		return "Idempotent";
	}

	/** 
     * @see nenok.lc.JoinTree#scale()
	 */
	
	public void scale() throws VAException {
        // scaling is redundant in idempotent architecture
	}

	/**
     * @see nenok.lc.JoinTree#createNode()
	 */
    
	public Node createNode() {
        return new IDNode(adapter);
	}
}
