package nenok.lc.ls;

import nenok.adapt.Adapter;
import nenok.adapt.Labeled;
import nenok.lc.LCException;
import nenok.lc.Node;
import nenok.va.Domain;

/**
 * This is the node class implementation that belongs to the 
 * Lauritzen-Spiegelhalter architecture.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 557 $<br/>$LastChangedDate: 2008-03-26 14:23:32 +0100 (Mi, 26 Mrz 2008) $
 */

public class LSNode extends Node {

    // Internal message cache:
	private int counter;
    private Labeled[] messages;
    
    /**
     * Constructor:
     * @param adapter The task factory for this node.
     */
    
    public LSNode(Adapter adapter) {
        super(adapter);
    }
                
    /**
     * @see nenok.lc.Node#collect()
     */
    
    public void collect() throws LCException { 
        
        // Test, if all parents have sent their message:
        if(counter < this.getParents().size()) {
            return;
        }
        
        // Compute new node content -> messages haven't been initialized for leaves.  
        if(getParents().size() > 0) {
            content = adapter.combine(content, messages);
                    
            // Throw messages away:
            messages = null;
        }
                           
        // Stop when reaching the root node:
        if(getChild() == null) {
            return;
        }
                      
        // Compute message for the child:
        Domain inter = Domain.intersection(content.label(), getChild().getDomain());        
        Labeled message = adapter.marginalize(content, inter);
        
        // Perform division:
        Labeled inv = adapter.inverse(message);                
        content = adapter.combine(content, inv);
                        
        // Send message to child:
        LSNode child = (LSNode)this.getChild();
        child.receiveParentMessage(message);
        child.collect();
    }

    /**
     * @see nenok.lc.Node#distribute()
     */
    
    public void distribute() throws LCException {
        for(Node parent : this.getParents()) {
            Labeled message = adapter.marginalize(content, Domain.intersection(content.label(), parent.getDomain()));
            ((LSNode)parent).receiveChildMessage(message);
            parent.distribute();
        }
    }
   
    /*
     * Helper methods:
     */
    
    /**
     * Method called by parent node to send message to this node.
     * @param message The message to send.
     */
    
    private void receiveParentMessage(Labeled message) {
        if(messages == null) {
            messages = new Labeled[getParents().size()];
        }
    	messages[counter++] = message;
    }
    
    /**
     * Method called by child node to send message to a parent
     * @param message The message to send.
     * @throws LCException Generic local computation exception.
     */
    
    private void receiveChildMessage(Labeled message) throws LCException {
        content = adapter.combine(content, message);
    }  
}
