package nenok.net;

import java.io.Serializable;

import nenok.va.Valuation;
import net.jini.id.Uuid;
import net.jini.id.UuidFactory;

/**
 * This class represents the communication envelope to store valuations.
 * Its main purpose is to attach identifiers to valuation objects without
 * modifying the valuation hierarchy.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class Envelope implements Serializable {
	
    private final Uuid envID;
    private final Valuation valuation;
    
    /**
     * Constructor:
     * @param valuation The valuation to be wrapped in this envelope.
     */
    
    public Envelope(Valuation valuation) {
        this(valuation, UuidFactory.generate());        
    }
    
    /**
     * Constructor:
     * @param valuation The valuation to be wrapped in this envelope.
     * @param envID The envelope's unique identifier.
     */
    
    public Envelope(Valuation valuation, Uuid envID) {
        this.envID = envID;
        this.valuation = valuation;
    }

    /**
     * @return Returns the envelope identifier.
     */
    
    public Uuid getID() {
        return envID;
    }
    
    /**
     * @return Returns the valuation wrapped in this envelope.
     */
    
    public Valuation getValuation() {
        return valuation;
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        return (o instanceof Envelope) && this.envID.equals(((Envelope)o).envID);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return envID.toString();
    }
}
