package nenok.va;

/**
 * This is a pre-implementation of the variable interface and represents 
 * variables with finite frames. Such variables are used (for example) 
 * in the context of semiring induced valuation algebras to represent 
 * configurations.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 559 $<br/>$LastChangedDate: 2008-03-26 14:25:18 +0100 (Mi, 26 Mrz 2008) $
 */

public class FiniteVariable implements Variable, Comparable {
	
	private final String name;
	private final String[] frame;
	
	/**
	 * Constructor:
	 * @param name The name of the variable.
	 * @param frame The variable's possible values.	 
	 */
	
	public FiniteVariable(String name, String[] frame) {
		this.name = name;
		this.frame = frame;
	}
	
	/**
	 * Returns the frame of this discrete variable.
	 * @return The variable's frame.
	 */
	
	public String[] getFrame() {
		return this.frame;
	}
	
    /**
     * Computes the number of configurations of the given variable array.
     * @param vars An array of variables with finite frames.
     * @return The number of configurations that can be built from the argument array.
     */
    
    public static int countConfigurations(FiniteVariable[] vars) {
        int size = 1;
        for (int i = 0; i < vars.length; i++) {
            size *= vars[i].getFrame().length;
        }
        return size;           
    }
	
	/**
	 * @see java.lang.Object#hashCode()
	 */
		
	public int hashCode() {
		return name.hashCode();
	}
	
	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	
	public boolean equals(Object o) {
	    return this.compareTo(o) == 0;
	}
	
	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	
	public int compareTo(Object arg) {
		return name.compareTo(((FiniteVariable)arg).name);
	}
	
	/**
	 * Converts the current variable to a string.
	 * @return The variable's string value.
	 */

	public String toString() {
		return name;
	}
}
