package nenok.va.sr;

/**
 * In a distributive C-semiring both operations are idempotent. Additionally, the unit 
 * element is absorbing, i.e. a + 1 = 1 for all semiring elements. Such a semiring is 
 * equivalent to a bounded distributive lattice. It is important to node that this 
 * interface does not extend {@link Optimization} since here we do not assume a 
 * (strictly monotonic) total order between semiring elements.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 560 $<br/>$LastChangedDate: 2008-03-26 14:37:05 +0100 (Mi, 26 Mrz 2008) $
 */

public interface ICSemiring extends SeparativeSemiring {
	
    /*
     * Marker interface:
     */
    
    /**
     * Implementor of the {@link SeparativeSemiring} interface.
     * 
     * @author Marc Pouly
     * @version 1.4
     */
    
    public class Implementor {
    	
    	// SINGLETON design pattern:
    	private static Implementor implementor = new Implementor();
    	
    	/**
    	 * Constructor:
    	 */
    	
    	private Implementor() {}
    	
    	/**
    	 * @return An instance of this implementor class.
    	 */
    	
    	public static Implementor getInstance() {
    		return implementor;
    	}

        /**
         * Pre-implementation of the {@link SeparativeSemiring} interface.
         * @param factor The factor whose inverse has to be computed.
         * @return The inverse of the factor.
         */
        
        public SeparativeSemiring inverse(ICSemiring factor) {
            return factor;
        }
    }
}
