package nenok.va.sr;

/**
 * Idempotent semiring that contain a total, natural order define a total ordering 
 * a + b = max(a,b). This natural semiring order is reflected by this interface which
 * therefore extends {@link Comparable}. Furthermore, this interface presupposes that
 * the ordering relation is strictly monotonic.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 560 $<br/>$LastChangedDate: 2008-03-26 14:37:05 +0100 (Mi, 26 Mrz 2008) $
 */

public interface Optimization extends Semiring, Comparable {
	
	/**
	 * Implementor class for the {@link Comparable} interface.
	 * 
	 * @author Marc Pouly
	 * @version $LastChangedRevision: 560 $<br/>$LastChangedDate: 2008-03-26 14:37:05 +0100 (Mi, 26 Mrz 2008) $
	 */
	
	public class Implementor {
		
		// SINGLETON design pattern:
		private static Implementor implementor = new Implementor();
		
		/**
		 * Constructor:
		 */
		
		private Implementor() {}
		
		/**
		 * @return An instance of this implementor class.
		 */
		
		public static Implementor getInstance() {
			return implementor;
		}
		
		/**
		 * @param elt1 The first element to be compared.
		 * @param elt2 The second element to be compared.
		 * @return {@link java.lang.Comparable#compareTo(java.lang.Object)}
		 */
		
		public int compare(Optimization elt1, Optimization elt2) {
			if(elt1.equals(elt2)) {
				return 0;
			}
			if(elt1.add(elt2).equals(elt1)) {
				return -1;
			}
			return 1;
		}
	}
}
